@@ -1,4 +1,4 @@
-/* $Id: prompt.c 4453 2009-12-02 03:36:22Z astyanax $ */
+/* $Id: prompt.c 4475 2010-01-17 05:30:22Z astyanax $ */
 /**************************************************************************
  *   prompt.c                                                             *
  *                                                                        *
@@ -229,7 +229,7 @@ int do_statusbar_input(bool *meta_key, bool *func_key, bool *have_shortcut,
 		f = sctofunc((sc *) s);
 		if (s->scfunc != 0 &&  s->execute == TRUE) {
 			*ran_func = TRUE;
-		    if (!ISSET(VIEW_MODE) || f->viewok)
+		    if (f && (!ISSET(VIEW_MODE) || (f->viewok)))
 		        iso_me_harder_funcmap(f->scfunc);
 		}
 		*finished = TRUE;
